package w6.ex1.toctou.ex;

/**
 * @invariant getStart() is before or at the same time as getEnd()
 */
public final class Period {
	private final Date start;
	private final Date end;

	/**
	 * @param start the beginning of the period
	 * @param end the end of the period; must not precede start
	 * @pre start.compareTo(end) <= 0
	 * @throws IllegalArgumentException@ if start is after end
	 * @throws NullPointerException if start or end is null
	 */
	public Period(Date start, Date end) {
		if (start.compareTo(end) > 0) {
			throw new IllegalArgumentException(start + " after " + end);
		}
		this.start = new Date(start.getTime());
		this.end = new Date(end.getTime());
	}

	public Date getStart() {
		return new Date(start.getTime());
	}

	public Date getEnd() {
		return new Date(end.getTime());
	}
}
