package w6.ex1.toctou.sol;

import java.util.Date;

/**
 * @invariant getStart() is before or at the same time as getEnd()
 */
public final class Period {
	private final Date start;
	private final Date end;

	/**
	 * @param start the beginning of the period
	 * @param end the end of the period; must not precede start
	 * @pre start.compareTo(end) <= 0
	 * @throws IllegalArgumentException@ if start is after end
	 * @throws NullPointerException if start or end is null
	 */
	public Period(Date start, Date end) {
		// To ensure that the invariant holds we need to *first* copy,
		// then test the *copies* for the invariant.
		// Between the invariant check and the copying, the Date objects may
		// have mutated in a concurrent setting.
		this.start = new Date(start.getTime());
		this.end = new Date(end.getTime());
		if (this.start.compareTo(this.end) > 0) {
			throw new IllegalArgumentException(this.start + " after " + this.end);
		}
		// Note that this solution doesn't work if we also add the
		// post-conditions (for the constructor)
		//   (after.this.start.compareTo(start) == 0 &&
		//    after.this.end.compareTo(end) == 0)
		// If we need this post condition, Date will have to be immutable.
	}

	public Date getStart() {
		return new Date(start.getTime());
	}

	public Date getEnd() {
		return new Date(end.getTime());
	}
}
