package w6.ex2.anonclass.sol;

import java.util.Comparator;
import java.util.List;
import java.util.LinkedList;
import java.util.Collections;

public class SortingFactory {
	private static final Comparator<String> numberOfCharactersComparatorInstance =
			new Comparator<String>() {
				@Override
				public int compare(String a, String b) {
					return a.length() - b.length();
				}
			};

	/**
	 * Returns a comparator for strings, ordering in ascending order of
	 * string length.
	 */
	public static Comparator<String> numberOfCharactersComparator() {
		return numberOfCharactersComparatorInstance;
	}

	/**
	 * Returns a comparator for strings, ordering in ascending order
	 * but ignores every character past prefixLength.
	 * 
	 * With prefixLength set to 2, "Pelle" and "Peter" should compare
	 * as equal but with prefixLength set to 3, "Pelle" would come before
	 * "Peter".
	 * 
	 * @param prefixLength
	 *            The maximum number of characters to use
	 *            when comparing.
	 */
	public static Comparator<String> stringPrefixComparator(
			final int prefixLength) {
		return new Comparator<String>() {
			@Override
			public int compare(String a, String b) {
				return a.substring(0, Math.min(prefixLength, a.length()))
							.compareTo(b.substring(0, Math.min(prefixLength, b.length())));
			}
		};
	}
	
	private static void printStringList(List<String> ss) {
		for(String s : ss) {
			System.out.println(s);
		}
	}
	
	public static void main(String[] args) {
		List<String> items = new LinkedList<String>();
		items.add("Peter");
		items.add("Pelle");
		items.add("Emil");
		items.add("Emilia");
		items.add("Emelie");
		items.add("Christer");
		System.out.println("Original order:");
		printStringList(items);
		
		System.out.println("\nSorted, using the 2-prefix");
		Collections.sort(items, stringPrefixComparator(2));
		printStringList(items);

		System.out.println("\nSorted, using the 3-prefix");
		Collections.sort(items, stringPrefixComparator(3));
		printStringList(items);

		System.out.println("\nSorted, using length only");
		Collections.sort(items, numberOfCharactersComparator());
		printStringList(items);
	}
}
